% MATLAB code
% File: asnlmg.m
%
function [Fy_doesntcause_x,Fx_doesntcause_y,py_doesnt_cause_x,px_doesnt_cause_y]=asnlmg(x,y,t1,t2,c1,c2,signx,signy)
%
% Asymmetric GRANGER-style causality test for Mackey-Glass model - 
%   testing for statistical significance of the nonlinear terms ONLY
%
% References:
% Hristu-Varsakelis, D. and Kyrtsou, C. (2008).
%   Evidence for nonlinear asymmetric causality in US inflation, metal and
%   stock returns, Discrete Dynamics in Nature and Society.
%   Available at: https://www.hindawi.com/journals/ddns/2008/138547/.
%   Doi:10.1155/2008/138547
% Hristu-Varsakelis, D. and Kyrtsou, C. (2010). 
%   Testing for Granger causality in the presence of chaotic dynamics, 
%   Brussels Economic Review, 53(2), 323-327. 
% Anoruo, E. (2011).
%   Testing for linear and nonlinear causality between crude oil price changes
%   and stock market returns. 
%   Int. J. of Economic Sciences and Applied Research, 4(3), 75-92.
% 
% INPUT:
% x, y  = the series to be tested; x = SP500percentage, y = COPchanges,
% t1,t2 = the delay values in the Mackey-Glass model
% c1,c2 = the exponents in the nonlinear terms of the M-G model:
%    X(t) = a(1,1)[X(t-t1)/{1+X(t-t1)^c1}-d(1,1)X(t-1)+a(1,2)[Y(t-t2)/{1+Y(t-t2)^c2}
%           -d(1,2)X_{t-1}+mu(t)
%    Y(t) = a(2,1)[X(t-t1)/{1+X(t-t1)^c1}-d(2,1)X(t-1)+a(2,2)[Y(t-t2)/{1+Y(t-t2)^c2}
%           -d(2,2)X_{t-1}+mu(t)
%    with mu(t) ~(i.i.d.) N(0,1)
%
% The variables signx,signy control which of the four possible asymetries
% to test for causality. +1 for "+", -1 for "-". That is, signx=1 causes the code
% to ignore any observations in which the returns of the x-variable are
% negative (-), or any observations which are related to negative x-returns
% via the nonlinear term on the right-hand side of the M-G model.
% EXACTLY ONE of the signx, signy inputs must be zero (e.g., if signx=0
% then we are testing for causality from y to x).
%
% OUTPUT: the F-values and p-values describing the statistical
% significance of the zero hypotheses (x doesn't cause y, y doesn't cause
% x). Low p-values suggest that the zero hypothesis is to be rejected. 
% See also Granger causality test.
%
% EXAMPLES: 
% [f1,f2,p1,p2]=asnlmg(x,y,1,1,2,2,-1,0); % tests whether negative
%                                         % x-series terms cause the y series
% [f1,f2,p1,p2]=asnlmg(x,y,1,3,2,2, 1,0); % tests whether positive
%                                         % x-series terms cause the y series
% [f1,f2,p1,p2]=asnlmg(x,y,1,1,2,2,0,-1); % tests whether negative
%                                         % y series terms cause the x series
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if(signx*signy ~=0)
    fprintf(1,'Error: only 1 of the last 2 arguments can be nonzero. Exiting.\n')
    return;
end


if (signx==0)
    fprintf('ASYMETRIC M-G causality TEST with NULL="Y!->X". ONLY f1, p1 values are valid\n');
else
    fprintf('ASYMETRIC M-G causality TEST w. NULL="X!->Y". ONLY f2, p2 values are valid\n');
end

kstart = max(t1,t2)+1;
N      = length(x);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Find best FIT parameters for the series
%
bigX    = [];
lengthX = 0;
indX    = [];

for k=kstart:length(x)
    % Exclude rows that contain outliers
    if (signy==0) % Test for X CAUSING THE Y SERIES
        if ((signx)*x(k-t1)>=0)
        %if ((signx)*x(k-t1)>0 && (signy)*y(k-t1)>0 && (signx)*x(k)>0 && (signy)*y(k)>0)
            bigX    = [bigX; x(k-t1)/(1+x(k-t1)^c1) x(k-1) y(k-t2)/(1+y(k-t2)^c2) y(k-1)];
            lengthX = lengthX+1;
            indX    = [indX,k];
        end
    else   % (signx=0 - Test for Y CAUSING THE X SERIES)
        if ((signy)*y(k-t2)>=0)
        % if ((signx)*x(k-t1)>0 && (signy)*y(k-t1)>0 && (signx)*x(k)>0 && (signy)*y(k)>0)
            bigX    = [bigX; x(k-t1)/(1+x(k-t1)^c1) x(k-1) y(k-t2)/(1+y(k-t2)^c2) y(k-1)];
            lengthX = lengthX+1;
            indX    = [indX,k];
        end
    end
end
%bigX = [bigX, ones(lengthX-kstart+1,1)];
%bigX = [bigX, ones(lengthX,1)];

indX;

lengthY = lengthX;
% Fit coefficients
est_ax = regress(x(indX),bigX);
est_ay = regress(y(indX),bigX);

%residuals
Rx     = x(indX)-bigX*est_ax;
Rxnorm = norm(Rx)^2;
Ry     = y(indX)-bigX*est_ay;
Rynorm = norm(Ry)^2;

%
% FIT FOR NULL HYPOTHESIS
% (remove y-data from bigX)
XX = bigX(:,[1,2,4]);
YY = bigX(:,[2,3,4]);

est_ax_null = regress(x(indX),XX);
Rxnull      = x(indX)-XX*est_ax_null;
Rxnullnorm  = norm(Rxnull)^2;

est_ay_null = regress(y(indX),YY);
Rynull      = y(indX)-YY*est_ay_null;
Rynullnorm  = norm(Rynull)^2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STATISTICS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE F STATISTIC
% How many coefficients do we restrict? always 2 in each equation
% How many unrestricted coefficients? always 4 in each equation
nlag   = kstart-1;
nrestr = 1;
nfree  = 3;
%nlag  = 3

Fy_doesntcause_x = ((Rxnullnorm-Rxnorm)/nrestr)/(Rxnorm/(N-nlag-nfree-1));
Fx_doesntcause_y = ((Rynullnorm-Rynorm)/nrestr)/(Rynorm/(N-nlag-nfree-1));

%p_val_x_caused_by_y = 1 - fpdf(Sxy,nlag,N - 2*nlag - 1);
py_doesnt_cause_x    = 1-cdf('F',Fy_doesntcause_x,nrestr,N-nlag-nfree - 1);

%p_val_y_caused_by_x = 1 - fpdf(Syx,nlag,N - 2*nlag - 1);
px_doesnt_cause_y    = 1-cdf('F',Fx_doesntcause_y,nrestr,N-nlag-nfree - 1);

