% MATLAB code
% File: mgparams.m
%
function [Rkurtosisx,Rskewx,Rschwarzx,Rschwarzy]=mgparams(x,y,t1,t2,c1,c2)
%
% This function estimates a Mackey-Glass model with parameters t1,t2, and
% c1,c2, given the series x and y, and returns a statistical evaluation of
% the fit. Its purpose is to facilitate the search for "good" model
% parameters.
%
% INPUT:
% x, y  = the series to be tested; x = SP500percentage, y = COPchanges,
% t1,t2 = the delay values in the Mackey-Glass model:
%    X(t) = a(1,1)[X(t-t1)/{1+X(t-t1)^c1}-d(1,1)X(t-1)+a(1,2)[Y(t-t2)/{1+Y(t-t2)^c2}
%           -d(1,2)X_{t-1}+mu(t)
%    Y(t) = a(2,1)[X(t-t1)/{1+X(t-t1)^c1}-d(2,1)X(t-1)+a(2,2)[Y(t-t2)/{1+Y(t-t2)^c2}
%           -d(2,2)X_{t-1}+mu(t)
%    with mu(t) ~(i.i.d.) N(0,1)
%
% c1,c2 = the exponents in the nonlinear terms of the M-G model
%
% OUTPUT: Kurtosis, Skewness and Schwarz criterion of the residuals.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
kstart = max(t1,t2)+1;
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Find best FIT parameters for the series
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bigX  = [];
for k=kstart:length(x)
    if 1+y(k-t2)== 0; 
       yy(k-t2) = 0;
    else  yy(k-t2) = y(k-t2)/(1+y(k-t2)^c2);
    end;             
    if 1+x(k-t1)== 0; 
       xx(k-t1) = 0; 
    else  xx(k) = x(k-t1)/(1+x(k-t1)^c1);    
    end;    
    bigX = [bigX; xx(k-t1) x(k-1) yy(k-t2) y(k-1)];
end

% Fit coefficients
est_ax = regress(x(kstart:length(x)),bigX);
est_ay = regress(y(kstart:length(y)),bigX);

% Residuals
Rx     = x(kstart:length(x))-bigX*est_ax;
Rxnorm = norm(Rx)^2;
Ry     = y(kstart:length(y))-bigX*est_ay;
Rynorm = norm(Ry)^2;

% Compute Kurtosis, Skewness and Schwarz statistic of the residuals
Rkurtosisx = kurtosis(Rx); 
Rskewx     = skewness(Rx); 
n          = (length(x)-kstart+1);
k          = 4; % 4 parameters to be estimated per equation
Rschwarzx  = log(Rxnorm/n)+k*log(n)/n;
Rschwarzy  = log(Rynorm/n)+k*log(n)/n;