% MATLAB code
% File: nlmg.m
%
function[Fx_notcaused_by_y,Fy_notcaused_by_x,py_doesnt_cause_x,px_doesnt_cause_y,est_ax,est_ay]=nlmg(x,y,t1,t2,c1,c2)
%
% GRANGER-style causality test for Mackey-Glass model - 
%     testing for statistical significance of the nonlinear terms ONLY
%
% References: 
% Hristu-Varsakelis, D. and Kyrtsou, C. (2008).
%   Evidence for nonlinear asymmetric causality in US inflation, metal and
%   stock returns, Discrete Dynamics in Nature and Society.
%   Available at: https://www.hindawi.com/journals/ddns/2008/138547/.
%   Doi:10.1155/2008/138547
% Hristu-Varsakelis, D. and Kyrtsou, C. (2010). 
%   Testing for Granger causality in the presence of chaotic dynamics, 
%   Brussels Economic Review, 53(2), 323-327. 
% Anoruo, E. (2011).
%   Testing for linear and nonlinear causality between crude oil price changes
%   and stock market returns. 
%   Int. J. of Economic Sciences and Applied Research, 4(3), 75-92.

% INPUT:
% x, y  = the series to be tested; x = SP500percentage, y = COPchanges,
% t1,t2 = the delay values in the Mackey-Glass model:
%    X(t) = a(1,1)[X(t-t1)/{1+X(t-t1)^c1}-d(1,1)X(t-1)+a(1,2)[Y(t-t2)/{1+Y(t-t2)^c2}
%           -d(1,2)X_{t-1}+mu(t)
%    Y(t) = a(2,1)[X(t-t1)/{1+X(t-t1)^c1}-d(2,1)X(t-1)+a(2,2)[Y(t-t2)/{1+Y(t-t2)^c2}
%           -d(2,2)X_{t-1}+mu(t)
%    with mu(t) ~(i.i.d.) N(0,1)
%
% c1,c2 = the exponents in the nonlinear terms of the M-G model
%
% OUTPUT: the F-values and p-values describing the statistical
% significance of the zero hypotheses (x doesn't cause y, y doesn't cause x). 
% Low p-values suggest that the zero hypothesis is to be rejected. 
% See also Granger causality test.
%
% est_ax, est_ay hold the estimated (best fit) coefficients of the M-G model.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
kstart = max(t1,t2)+1;
N      = length(x)-kstart+1;  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Find best FIT parameters for the series
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bigX = [];
for k=kstart:length(x)
    if 1+y(k-t2)== 0; 
       yy(k-t2) = 0;
    else  yy(k-t2) = y(k-t2)/(1+y(k-t2)^c2);
    end;             
    if 1+x(k-t1)== 0; 
       xx(k-t1) = 0; 
    else  xx(k) = x(k-t1)/(1+x(k-t1)^c1);    
    end;    
    bigX = [bigX; xx(k-t1) x(k-1) yy(k-t2) y(k-1)];
end

% Fit coefficients
est_ax = regress(x(kstart:length(x)),bigX);
est_ay = regress(y(kstart:length(y)),bigX);

% Residuals
Rx     = x(kstart:length(x))-bigX*est_ax;
Rxnorm = norm(Rx)^2;
Ry     = y(kstart:length(y))-bigX*est_ay;
Rynorm = norm(Ry)^2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FIT FOR NULL HYPOTHESIS
% (delete y-data from bigX)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%XX = bigX(:,[1,2,4,5]); % REGRESS WITH CONSTANTS (TESTING)
%YY = bigX(:,[2,3,4,5]);

%XX = bigX(:,1);
%YY = bigX(:,2);

XX = bigX(:,[1,2,4]);
YY = bigX(:,[2,3,4]);

est_ax_null = regress(x(kstart:length(x)),XX);
Rxnull      = x(kstart:length(x))-XX*est_ax_null;
Rxnullnorm  = norm(Rxnull)^2;

est_ay_null = regress(y(kstart:length(y)),YY);
Rynull      = y(kstart:length(y))-YY*est_ay_null;
Rynullnorm  = norm(Rynull)^2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STATISTICS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% COMPUTE F STATISTIC
% How many coefficients do we restrict? always 1 in each equation
% How many unrestricted coefficients? always 3 in each equation
nlag   = kstart-1;
nrestr = 1;
nfree  = 3;
%nlag  = 3;
Fx_notcaused_by_y = ((Rxnullnorm-Rxnorm)/nrestr)/(Rxnorm/(N-nlag-nfree-1));
Fy_notcaused_by_x = ((Rynullnorm-Rynorm)/nrestr)/(Rynorm/(N-nlag-nfree-1));

%p_val_x_caused_by_y = 1 - fpdf(Sxy,nlag,N - 2*nlag - 1);
py_doesnt_cause_x = 1-cdf('F',Fx_notcaused_by_y,nrestr,N-nlag-nfree - 1);

%p_val_y_caused_by_x = 1 - fpdf(Syx,nlag,N - 2*nlag - 1);
px_doesnt_cause_y    = 1-cdf('F',Fy_notcaused_by_x,nrestr,N-nlag-nfree - 1);