% MATLAB code
% File: nlmg_kyrtsou.m
%
function [Fx_notcaused_by_y,py_doesnt_cause_x,est_ax,est_ay,LLF,SBC]=nlmg_kyrtsou(x,y,t1,t2,c1,c2)
%
% GRANGER-style causality test for Mackey-Glass model - 
%      testing for statistical significance of the nonlinear terms ONLY
%
% References: 
% Hristu-Varsakelis, D. and Kyrtsou, C. (2010). 
%   Testing for Granger causality in the presence of chaotic dynamics, 
%   Brussels Economic Review, 53(2), 323-327. 
% Hristu-Varsakelis, D. and Kyrtsou, C. (2008).
%   Evidence for nonlinear asymmetric causality in US inflation, metal and
%   stock returns, Discrete Dynamics in Nature and Society.
%   Available at: https://www.hindawi.com/journals/ddns/2008/138547/.
%   Doi:10.1155/2008/138547
% Anoruo, E. (2011).
%   Testing for linear and nonlinear causality between crude oil price changes
%   and stock market returns. 
%   Int. J. of Economic Sciences and Applied Research, 4(3), 75-92.
%
% v2: added computation of residuals, made the function "1-sided" (perform 1
% test only), computes LLF function as well.
%
% INPUT:
% x, y  = the series to be tested
% t1,t2 = the delay values in the Mackey-Glass model
% c1,c2 = the exponents in the nonlinear terms of the M-G model
%
% OUTPUT: the F-values and p-values describing the statistical
% significance of the zero hypotheses (x doesn't cause y, y doesn't cause x). 
% Low p-values suggest that the zero hypothesis is to be rejected.
% See also Granger causality test.
%
% est_ax holds the estimated (best fit) coefficients of the M-G
% model.
%
% Copyright 2007, Dimitrios Hristu-Varsakelis.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

kstart = max(t1,t2)+1;
N      = length(x)-kstart+1;  %x

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Find best FIT parameters for the series
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bigX = [];
bigY = [];
for k=kstart:length(x)
    bigX = [bigX; x(k-t1)/(1+x(k-t1)^c1) x(k-1) y(k-t2)/(1+y(k-t2)^c2) y(k-1)];
end

if (size(bigX,1)>1)  
    % Fit coefficients
    est_ax = regress(x(kstart:length(x)),bigX);
    est_ay = regress(y(kstart:length(y)),bigX);  
    
    % Residuals
    Rx     = x(kstart:length(x))-bigX*est_ax;
    Rxnorm = norm(Rx)^2;
    Ry     = y(kstart:length(y))-bigX*est_ay;
    Rynorm = norm(Ry)^2;
    
    % Are the residuals normally distributed?
    
    % jbtest(Rx)
    % lillietest(Rx)
    % kstest(Rx)
    % mean(Rx)
    % var(Rx)
    
    % Now, compute LLF
    n   = length(x);
    Res = [Rx Ry];
    sig = cov(Res);
    
    % Zi    = [x(kstart:length(x)) y(kstart:length(y))];
    % Zihat = [bigX*est_ax, bigX*est_ay];
        
    % LLF   = -n/2*log((2*pi)^2)-n/2*log(det(sig));
    % for q=1:length(Zi)
    %    LLF = LLF-1/2*((Zi(q,:)-Zihat(q,:))*inv(sig)*(Zi(q,:)-Zihat(q,:))');
    % end
    
    % LLF1 = -n/2*log(2*pi)-n*log(std(Rx))-1/(2*var(Rx))*(x(kstart:length(x))'*x(kstart:length(x))+est_ax'*bigX'*bigX*est_ax-2*est_ax'*bigX'*x(kstart:length(x)));
    % LLF2 = -n/2*log(2*pi)-n*log(std(Ry))-1/(2*var(Ry))*(y(kstart:length(x))'*y(kstart:length(y))+est_ay'*bigX'*bigX*est_ay-2*est_ay'*bigX'*y(kstart:length(y)));
    
    % Note: To match EViews definition of LLF...
    nvars = 2;
    LLF   = -n/2*(nvars*(1+log(2*pi))+log(det(sig)));
    
    % for q=1:length(Zi)
    %  LLF = LLF-n/2*log(Rx'*Rx/n)-n/2*log(Ry'*Ry/n);
    % end

    
    % NEW WAY OF COMPUTING SBC
    % kstart = max(t1,t2)+1;
    np       = nvars*kstart+1;
    Chat     = 0;
    for i=1:length(Rx)
        Chat = Chat+Res(i,:)'*Res(i,:);
    end
    Delp  = Chat/(length(Rx)-1);
    lp    = log(det(Delp));
    % SBC = lp/nvars-(1-np/N)*log(N);
    SBC   = lp+8*log(N)/N;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % FIT FOR NULL HYPOTHESIS
    % (delete y-data from bigX)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % XX = bigX(:,[1,2,4,5]); % REGRESS WITH CONSTANTS (TESTING)
    
    % XX = bigX(:,1);
    
    XX = bigX(:,[1,2,4]);
    
    est_ax_null = regress(x(kstart:length(x)),XX);
    Rxnull      = x(kstart:length(x))-XX*est_ax_null;
    Rxnullnorm  = norm(Rxnull)^2;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % STATISTICS
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % COMPUTE F STATISTIC
    % How many coefficients do we restrict? always 1 in each equation
    % How many unrestricted coefficients? always 3 in each equation
    nlag   = kstart-1;
    nrestr = 1;
    nfree  = 3;
    %nlag  = 3
    %Fx_notcaused_by_y = ((Rxnullnorm-Rxnorm)/nrestr)/(Rxnorm/(N-nlag-nfree-1));
    Fx_notcaused_by_y  = ((Rxnullnorm-Rxnorm)/nrestr)/(Rxnorm/(N-nfree-nrestr-1));
    
    % p_val_x_caused_by_y = 1 - fpdf(Sxy,nlag,N - 2*nlag - 1)
    % py_doesnt_cause_x=1-cdf('F',Fx_notcaused_by_y,nrestr,N-nlag-nfree - 1);
    py_doesnt_cause_x = 1-cdf('F',Fx_notcaused_by_y,nrestr,N-nfree-nrestr-1);
        
else % Not enough data
    
    Fx_notcaused_by_y = 0;
    py_doesnt_cause_x = 1;
    est_ax = [];
    est_ay = [];   
end